package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (试剂耗材类)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材类) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/reagent_consumables" )
@Api(value = "reagent_consumables", tags = "(试剂耗材类)管理")
public class ReagentConsumablesController {

    private final  ReagentConsumablesService reagentConsumablesService;

	/**
	 * 通过id查询(试剂耗材类)
	 * @param reagentConsumablesId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("" )
//	@PreAuthorize("@pms.hasPermission('reagent_reagent_consumables_get')" )
	public R<ReagentConsumables> getById(String reagentConsumablesId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ReagentConsumables reagentConsumables = reagentConsumablesService.getById(reagentConsumablesId);
		return R.ok(reagentConsumables);
		//return R.ok(reagentConsumablesService.getById(reagentConsumablesId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param reagentConsumables (试剂耗材类)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumables_get')" )
    public R<IPage<ReagentConsumables>> getReagentConsumablesPage(Page<ReagentConsumables> page, ReagentConsumables reagentConsumables, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<ReagentConsumables> reagentConsumablesSList = reagentConsumablesService.page(page, Wrappers.<ReagentConsumables>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(reagentConsumablesSList);
//		return R.ok(reagentConsumablesService.page(page, Wrappers.query(reagentConsumables)));
    }


    /**
     * 新增(试剂耗材类)
     * @param reagentConsumables (试剂耗材类)
     * @return R
     */
    @ApiOperation(value = "新增(试剂耗材类)", notes = "新增(试剂耗材类)")
    @SysLog("新增(试剂耗材类)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumables_add')" )
    public R<ReagentConsumables> postAddObject(@RequestBody ReagentConsumables reagentConsumables, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        reagentConsumables.setReagentConsumableId(IdWorker.get32UUID().toUpperCase());
		if (reagentConsumablesService.save(reagentConsumables)) {
			return R.ok(reagentConsumables, "对象创建成功");
		}
		else {
			return R.failed(reagentConsumables, "对象创建失败");
		}
    }

    /**
     * 修改(试剂耗材类)
     * @param reagentConsumables (试剂耗材类)
     * @return R
     */
    @ApiOperation(value = "修改(试剂耗材类)", notes = "修改(试剂耗材类)")
    @SysLog("修改(试剂耗材类)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumables_edit')" )
    public R<ReagentConsumables> putUpdateById(@RequestBody ReagentConsumables reagentConsumables, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (reagentConsumablesService.updateById(reagentConsumables)) {
			return R.ok(reagentConsumables, "保存对象成功");
		}
		else {
			return R.failed(reagentConsumables, "保存对象失败");
		}
    }

    /**
     * 通过id删除(试剂耗材类)
     * @param reagentConsumablesId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(试剂耗材类)", notes = "通过id删除(试剂耗材类)")
    @SysLog("通过id删除(试剂耗材类)" )
    @DeleteMapping("/{reagentConsumablesId}" )
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumables_del')" )
    public R<ReagentConsumables> deleteById(@PathVariable String reagentConsumablesId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ReagentConsumables oldReagentConsumables = reagentConsumablesService.getById(reagentConsumablesId);

		if (reagentConsumablesService.removeById(reagentConsumablesId)) {
			return R.ok(oldReagentConsumables, "对象删除成功");
		}
		else {
			return R.failed(oldReagentConsumables, "对象删除失败");
		}

    }

}
