package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.StandardMaterialApprovalFormDTO;
import digital.laboratory.platform.reagent.entity.StandardMaterialApprovalForm;
import digital.laboratory.platform.reagent.entity.SupplierInformation;
import digital.laboratory.platform.reagent.mapper.StandardMaterialApprovalFormMapper;
import digital.laboratory.platform.reagent.service.StandardMaterialApprovalFormService;
import digital.laboratory.platform.reagent.service.SupplierInformationService;
import digital.laboratory.platform.reagent.vo.StandardMaterialApprovalFormVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * (标准物质停用/报废销毁/恢复/降级使用审批表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质停用/报废销毁/恢复/降级使用审批表) 服务实现类
 */
@Service
public class StandardMaterialApprovalFormServiceImpl extends ServiceImpl<StandardMaterialApprovalFormMapper, StandardMaterialApprovalForm> implements StandardMaterialApprovalFormService {

    @Autowired
    private StandardMaterialApprovalFormService standardMaterialApprovalFormService;

    @Autowired
    private SupplierInformationService supplierInformationService;
    @Override
    public StandardMaterialApprovalForm addById(StandardMaterialApprovalFormDTO standardMaterialApprovalFormDTO, DLPUser dlpUser){

        StandardMaterialApprovalForm standardMaterialApprovalForm = new StandardMaterialApprovalForm();

        BeanUtils.copyProperties(standardMaterialApprovalFormDTO,standardMaterialApprovalForm);

        if (standardMaterialApprovalFormService.save(standardMaterialApprovalForm)){
            return standardMaterialApprovalForm;
        }else throw new RuntimeException(String.format("保存失败"));
    }

    @Override
    public StandardMaterialApprovalForm editById(StandardMaterialApprovalFormDTO standardMaterialApprovalFormDTO){

        StandardMaterialApprovalForm  standardMaterialApprovalForm = standardMaterialApprovalFormService.getById(standardMaterialApprovalFormDTO.getStandardMaterialApprovalFormId());

        BeanUtils.copyProperties(standardMaterialApprovalFormDTO,standardMaterialApprovalForm);

        if (standardMaterialApprovalFormService.updateById(standardMaterialApprovalForm)){
            return standardMaterialApprovalForm;
        }else throw new RuntimeException(String.format("保存失败"));
    }

    @Override
    public StandardMaterialApprovalForm commitById(StandardMaterialApprovalFormDTO standardMaterialApprovalFormDTO, DLPUser dlpUser){

        if (standardMaterialApprovalFormDTO.getStandardMaterialApprovalFormId().equals("")){

            StandardMaterialApprovalForm standardMaterialApprovalForm = addById(standardMaterialApprovalFormDTO, dlpUser);

            return standardMaterialApprovalForm;
        }else {

            StandardMaterialApprovalForm byId = standardMaterialApprovalFormService.getById(standardMaterialApprovalFormDTO.getStandardMaterialApprovalFormId());

            BeanUtils.copyProperties(standardMaterialApprovalFormDTO,byId);

            if (standardMaterialApprovalFormService.updateById(byId)){
                return byId;
            }else throw new RuntimeException(String.format("提交失败"));
        }
    }

    @Override
    public IPage<StandardMaterialApprovalFormVO> getVOPage(IPage<StandardMaterialApprovalForm> page, QueryWrapper<StandardMaterialApprovalForm> qw){

        IPage<StandardMaterialApprovalFormVO> voPage = baseMapper.getVOPage(page, qw);

        return voPage;
    }
    @Override
    public StandardMaterialApprovalFormVO getVO(String standardMaterialApprovalFormId){

        StandardMaterialApprovalForm byId = standardMaterialApprovalFormService.getById(standardMaterialApprovalFormId);

        StandardMaterialApprovalFormVO standardMaterialApprovalFormVO = new StandardMaterialApprovalFormVO();

        BeanUtils.copyProperties(byId,standardMaterialApprovalFormVO);

        SupplierInformation byId1 = supplierInformationService.getById(standardMaterialApprovalFormVO.getManufacturerId());

        standardMaterialApprovalFormVO.setManufacturerName(byId1.getSupplierName());

        return standardMaterialApprovalFormVO;
    }

    @Override//一级审核
    public StandardMaterialApprovalForm auditPrimary(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser){

        StandardMaterialApprovalForm byId = standardMaterialApprovalFormService.getById(auditAndApproveDTO.getUuId());

        byId.setAuditOpinionOfPrimary(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditResultOfPrimary(auditAndApproveDTO.getAuditResult());
        byId.setPrimaryAuditorId(dlpUser.getId());
        byId.setAuditTimeOfPrimary(LocalDateTime.now());

        if (standardMaterialApprovalFormService.updateById(byId)){
            return byId;
        }else throw new RuntimeException(String.format("审核失败"));
    }

    @Override//二级审核
    public StandardMaterialApprovalForm auditSecondary(AuditAndApproveDTO auditAndApproveDTO,DLPUser dlpUser){

        StandardMaterialApprovalForm byId = standardMaterialApprovalFormService.getById(auditAndApproveDTO.getUuId());

        byId.setAuditOpinionOfSecondary(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditResultOfSecondary(auditAndApproveDTO.getAuditResult());
        byId.setSecondaryAuditorId(dlpUser.getId());
        byId.setAuditTimeOfSecondary(LocalDateTime.now());

        if (standardMaterialApprovalFormService.updateById(byId)){
            return byId;
        }else throw new RuntimeException(String.format("审核失败"));
    }

    @Override//审批
    public StandardMaterialApprovalForm approveById(AuditAndApproveDTO auditAndApproveDTO,DLPUser dlpUser){

        StandardMaterialApprovalForm byId = standardMaterialApprovalFormService.getById(auditAndApproveDTO.getUuId());

        byId.setOpinionOfApproval(auditAndApproveDTO.getAuditOpinion());
        byId.setResultOfApproval(auditAndApproveDTO.getAuditResult());
        byId.setApproverId(dlpUser.getId());
        byId.setApprovalOfTime(LocalDateTime.now());

        if (standardMaterialApprovalFormService.updateById(byId)){
            return byId;
        }else throw new RuntimeException(String.format("审批失败"));
    }
}
